function [intersection] = lin_intersect(m1, b1, m2, b2, varargin)
%Returns the intersection of two lines based on their slopes and intercepts
%   Inputs:
%   m1: [1x1] double - the slope of line 1
%   b1: [1x1] double - the y intercept of line 1
%   m2: [1x1] double - the slope of line 2
%   b2: [1x1] double - the y intercept of line 2
%
%   Optional arguments:
%   plot_flag: [1x1] double - 1 if we want to plot outputs, 0 otherwise
%
%   Outputs:
%   intersection: the x and y coordinates of the intersection

% Below is an if statement. If the conditions specified after the if/elseif
% statements are true, the lines of code following will be executed.
% Otherwise, the lines of code after "else" will be run. Notice we use '=='
% not '=' to compare if values are the same. We could also use
% >=, <=, <, or >, to test for other relationships, but we won't here.

if nargin == 4     % no optional argments are entered
    plot_flag = 1; % defaults to plotting the output
elseif nargin == 5 % one optional argument was entered
    plot_flag = varargin{1}; % takes the value of the fifth argument
else
    warning('Initializing with default values.')
    m1 = 2; m2 = -5; b1 = -10; b2 = 20; plot_flag = 1;
end

if m1 == m2 && b1 == b2 %check if both slopes and y intercepts are the same
    warning('These lines are the same! They always intersect.')
    intersection = [nan,nan];
elseif m1 == m2 %check if only the slopes are the same
    warning('These lines are the parallel! They never intersect.')
    intersection = [nan,nan];
else %Calculate the intersection!
    
    x_int = (b2-b1)./(m1-m2);
    y_int = m1.*(x_int)+b1;
    
    % This returns the
    intersection = [x_int, y_int];
end

if plot_flag == 1 % optional argument that allows plotting
    %% Preparing to plot the lines
    % create an array of x values +- 10 units from the intersection spaced
    % by .1
    if ~isnan(intersection(1)) % checks if there is a valid intersection
        xmin = intersection(1) - 10;
        xmax = intersection(1) + 10;
    else
        xmin = -20 ; xmax = 20;% if not, defaults to [-20, 20]
    end
    x_range = xmin:.1:xmax;
    
    % return the y value at each of the sampled x values
    y1 = m1.*x_range+b1;
    y2 = m2.*x_range+b2;
    
    %% Plotting the lines
    figure % This creates a new figure
    plot(x_range,y1,'linewidth',3); % This plots our sampled x and y values
    %for line 1
    hold on % This keeps the next plot command from deleting the old graph
    plot(x_range,y2,'linewidth',3); % This plots our second line
    
    % Plot the intersection we found as a black square with MarkerSize = 10
    plot(intersection(1), intersection(2), 'ks', 'MarkerSize',10,...
        'MarkerFaceColor','k');
    
    % Set the limits of our graph
    xlim([min(x_range),max(x_range)]);
    ylim([min([y1 y2]),max([y1,y2])]);
    
    % Create a legend for our data
    legend('Line 1','Line 2','Intersection');
    
    % Now some labels for our axes
    xlabel('X Values (unitless)');
    ylabel('Y Values (unitless)');
    title('Intersection of Two Lines');
    
    % The graph could still look a little neater. Let's modify the plot
    % appearance (gcf is get current figure, gca is get current axis)
    set(gcf,'Color','w');
    set(gca,'Color','w','XColor','k','YColor','k','FontSize', 12, 'Box',...
        'on', 'LineWidth', 3.0);
    set(get(gca,'Title'),'Color','k','FontSize',18,'FontWeight','bold');
    set(get(gca,'Xlabel'),'FontSize',16,'FontWeight','bold');
    set(get(gca,'Ylabel'),'FontSize',16,'FontWeight','bold');
    %set(gcf,'Color','k');
    grid on
    
    hold off
    
else
end
end

